uint8_t settingSterilizationTimeIdx = 1; //in index. 1 = default
uint8_t settingUvTimeIdx = 2; //in index. 0 = off
uint8_t settingPostUvTimeIdx = 1; //in index. 0 = off
bool settingAutoMode = false;
bool settingAutoRefillMode = false;

String settingApPassword = "12345678";

int settingTick = 0;
int settingTickPrev = 0;

bool GetSettingAutoMode(){
  return settingAutoMode;
}

void SetSettingAutoMode(bool v){
  if (v!=settingAutoMode){
    settingAutoMode = v;
    settingTick++;
  }
}

bool GetSettingAutoRefillMode(){
  return settingAutoRefillMode;
}

void SetSettingAutoRefillMode(bool v){
  if (v!=settingAutoRefillMode){
    settingAutoRefillMode = v;
    settingTick++;
  }
}

uint8_t GetSterilizationTimeIndex(){
  return settingSterilizationTimeIdx;
}

void SetSterilizationTimeIndex(int v){
  if (v!=settingSterilizationTimeIdx){
    settingSterilizationTimeIdx = v;
    settingTick++;
  }
}

uint8_t GetUVTimeIndex(){
  return settingUvTimeIdx;
}

void SetUVTimeIndex(int v){
  if (v!=settingUvTimeIdx){
    settingUvTimeIdx = v;
    settingTick++;
  }
}

uint8_t GetPostUVTimeIndex(){
  return settingUvTimeIdx;
}

void SetPostUVTimeIndex(int v){
  if (v!=settingPostUvTimeIdx){
    settingPostUvTimeIdx = v;
    settingTick++;
  }
}

void GenerateNewApPassword(){
  //generate character with letter and number
  Serial.println("not impelemented");
  //................
  settingTick++;
}

String GetApPassword(){
  return settingApPassword;
}

